@@ -76,7 +76,12 @@ ConfigDialog::ConfigDialog(CsoundQt *parent, Options *options, ConfigLists *conf
 			midiInterfaceComboBox->addItem(QString::fromStdString(midiin.getPortName(i)), QVariant(i));
 		}
 	}
+
+#ifdef QCS_OLD_RTMIDI
 	catch (RtError &error) {
+#else
+	catch (RtMidiError &error) {
+#endif
 		// Handle the exception here
 		error.printMessage();
 	}
@@ -86,7 +91,11 @@ ConfigDialog::ConfigDialog(CsoundQt *parent, Options *options, ConfigLists *conf
 			midiOutInterfaceComboBox->addItem(QString::fromStdString(midiout.getPortName(i)), QVariant(i));
 		}
 	}
+#ifdef QCS_OLD_RTMIDI
 	catch (RtError &error) {
+#else
+	catch (RtMidiError &error) {
+#endif
 		// Handle the exception here
 		error.printMessage();
 	}
@@ -235,6 +244,7 @@ ConfigDialog::ConfigDialog(CsoundQt *parent, Options *options, ConfigLists *conf
 	csoundExecutableLineEdit->setText(m_options->csoundExecutable);
 	logFileLineEdit->setText(m_options->logFile);
 	sdkLineEdit->setText(m_options->sdkDir);
+	templateLineEdit->setText(m_options->templateDir);
 
 	TerminalLineEdit->setText(m_options->terminal);
 	browserLineEdit->setText(m_options->browser);
@@ -267,6 +277,7 @@ ConfigDialog::ConfigDialog(CsoundQt *parent, Options *options, ConfigLists *conf
 	connect(pythonDirToolButton, SIGNAL(clicked()), this, SLOT(browsePythonDir()));
 	connect(logFileToolButton, SIGNAL(clicked()), this, SLOT(browseLogFile()));
 	connect(sdkToolButton, SIGNAL(clicked()), this, SLOT(browseSdkDir()));
+	connect(templateToolButton, SIGNAL(clicked()), this, SLOT(browseTemplateDir()));
 
 	//  connect(this, SIGNAL(changeFont()), parent, SLOT(changeFont()));
 	connect(audioInputToolButton, SIGNAL(released()), this, SLOT(selectAudioInput()));
@@ -422,6 +433,7 @@ void ConfigDialog::accept()
 	m_options->csoundExecutable = csoundExecutableLineEdit->text();
 	m_options->logFile = logFileLineEdit->text();
 	m_options->sdkDir = sdkLineEdit->text();
+	m_options->templateDir = templateLineEdit->text();
 
 	m_options->terminal = TerminalLineEdit->text();
 	m_options->browser = browserLineEdit->text();
@@ -568,6 +580,12 @@ void ConfigDialog::browseSdkDir()
 	sdkLineEdit->setText(m_options->sdkDir);
 }
 
+void ConfigDialog::browseTemplateDir()
+{
+	browseDir(m_options->templateDir);
+	templateLineEdit->setText(m_options->templateDir);
+}
+
 void ConfigDialog::selectAudioInput()
 {
 	QList<QPair<QString, QString> > deviceList
