@@ -55,15 +55,15 @@ CsoundEngine::CsoundEngine(ConfigLists *configlists) :
 	ud->mouseValues.resize(6); // For _MouseX _MouseY _MouseRelX _MouseRelY _MouseBut1 and _MouseBut2 channels
 	ud->wl = NULL;
 	ud->midiBuffer = NULL;
+	ud->virtualMidiBuffer = NULL;
 	ud->playMutex = &m_playMutex;
 #ifdef QCS_PYTHONQT
 	ud->m_pythonCallback = "";
 #endif
 
-	m_recording = false;
 	m_consoleBufferSize = 0;
-	m_recBufferSize = 4096;
-	m_recBuffer = (MYFLT *) calloc(m_recBufferSize, sizeof(MYFLT));
+
+	m_recording = false;
 
 #ifndef QCS_DESTROY_CSOUND
 	// Create only once
@@ -104,7 +104,6 @@ CsoundEngine::~CsoundEngine()
 	csoundDestroy(ud->csound);
 #endif
 	delete ud;
-	free(m_recBuffer);
 }
 
 #ifndef CSOUND6
@@ -289,9 +288,10 @@ int CsoundEngine::midiReadCb(CSOUND *csound, void *ud_, unsigned char *buf, int
 {
 	CsoundUserData *ud = (CsoundUserData *) ud_;
 	Q_UNUSED(csound);
-	int count;
+	int count, countVirtual;
 	count = csoundReadCircularBuffer(ud->csound, ud->midiBuffer, buf, nBytes);
-	return count;
+	countVirtual = csoundReadCircularBuffer(ud->csound, ud->virtualMidiBuffer, buf + count, nBytes - count);
+	return count + countVirtual;
 }
 
 int CsoundEngine::midiInCloseCb(CSOUND *csound, void *ud)
@@ -354,6 +354,13 @@ void CsoundEngine::queueMidiIn(std::vector< unsigned char > *message)
 	}
 }
 
+void CsoundEngine::queueVirtualMidiIn(std::vector< unsigned char > &message)
+{
+	if (ud->virtualMidiBuffer) {
+		csoundWriteCircularBuffer(ud->csound, ud->virtualMidiBuffer, message.data(), message.size()* sizeof(unsigned char));
+	}
+}
+
 void CsoundEngine::sendMidiOut(QVector<unsigned char> &message)
 {
 
@@ -709,34 +716,31 @@ void CsoundEngine::pause()
 
 int CsoundEngine::startRecording(int sampleformat, QString fileName)
 {
-	const int channels=ud->numChnls;
-	const int sampleRate=ud->sampleRate;
-	int format = SF_FORMAT_WAV;
-	switch (sampleformat) {
-	case 0:
-		format |= SF_FORMAT_PCM_16;
-		break;
-	case 1:
-		format |= SF_FORMAT_PCM_24;
-		break;
-	case 2:
-		format |= SF_FORMAT_FLOAT;
-		break;
-	}
-	qDebug("start recording: %s", fileName.toLocal8Bit().constData());
-	m_outfile = new SndfileHandle(fileName.toLocal8Bit(), SFM_WRITE, format, channels, sampleRate);
+	qDebug("start recording (%i-bit samples): %s",
+		   (sampleformat + 2) * 8,
+		   fileName.toLocal8Bit().constData());
 	// clip instead of wrap when converting floats to ints
-	m_outfile->command(SFC_SET_CLIPPING, NULL, SF_TRUE);
-	samplesWritten = 0;
-	m_recording = true;
-
-	recordTimer.singleShot(20, this, SLOT(recordBuffer()));
+#ifdef PERFTHREAD_RECORD
+	// perfthread record API is only available with csound >= 6.04
+	if (ud->perfThread) {
+		m_recording = true;
+		ud->perfThread->Record(fileName.toLocal8Bit().constData(),
+							   (sampleformat + 2) * 8);
+	}
+#endif
 	return 0;
 }
 
 void CsoundEngine::stopRecording()
 {
-	m_recording = false;  // Will be processed on next record buffer
+	m_recording = false;
+
+#ifdef	PERFTHREAD_RECORD
+	if (ud->perfThread) {
+		ud->perfThread->StopRecord();
+	}
+	qDebug("Recording stopped.");
+#endif
 }
 
 void CsoundEngine::queueEvent(QString eventLine, int delay)
@@ -786,6 +790,9 @@ int CsoundEngine::runCsound()
 #ifdef CSOUND6
 	ud->midiBuffer = csoundCreateCircularBuffer(ud->csound, 1024, sizeof(unsigned char));
 	Q_ASSERT(ud->midiBuffer);
+	ud->virtualMidiBuffer = csoundCreateCircularBuffer(ud->csound, 1024, sizeof(unsigned char));
+	Q_ASSERT(ud->virtualMidiBuffer);
+//	csoundFlushCircularBuffer(ud->csound, ud->midiBuffer);
 #endif
 #endif
 
@@ -793,11 +800,13 @@ int CsoundEngine::runCsound()
 	if(m_debugging) {
 		csoundDebuggerInit(ud->csound);
 		foreach(QVariantList bp, m_startBreakpoints) {
-			Q_ASSERT(bp.size() == 3);
+			Q_ASSERT(bp.size() > 1);
 			if (bp[0].toString() == "instr") {
+				Q_ASSERT(bp.size() == 3);
 				csoundSetInstrumentBreakpoint(ud->csound, bp[1].toDouble(), bp[2].toInt());
 			} else if (bp[0].toString() == "line") {
-//				csoundSetBreakpoint(ud->csound, bp[1].toInt(), bp[2].toInt());
+				Q_ASSERT(bp.size() == 4);
+				csoundSetBreakpoint(ud->csound, bp[2].toInt(), bp[1].toInt(), bp[3].toInt());
 			} else {
 				qDebug() << "CsoundEngine::setStartingBreakpoints wrong breakpoint format";
 			}
@@ -821,7 +830,6 @@ int CsoundEngine::runCsound()
 
 #ifdef CSOUND6
 	csoundCreateMessageBuffer(ud->csound, 0);
-	csoundFlushCircularBuffer(ud->csound, ud->midiBuffer);
 #else
 	// Message Callbacks must be set before compile, otherwise some information is missed
 	csoundSetMessageCallback(ud->csound, &CsoundEngine::messageCallbackThread);
@@ -973,6 +981,9 @@ void CsoundEngine::cleanupCsound()
 #ifdef QCS_DESTROY_CSOUND
 
 		csoundDestroyCircularBuffer(ud->csound, ud->midiBuffer);
+		ud->midiBuffer = NULL;
+		csoundDestroyCircularBuffer(ud->csound, ud->virtualMidiBuffer);
+		ud->virtualMidiBuffer = NULL;
 		csoundDestroy(ud->csound);
 		ud->csound = NULL;
 #else
@@ -1076,7 +1087,7 @@ void CsoundEngine::setupChannels()
 
 void CsoundEngine::messageListDispatcher(void *data)
 {
-	qDebug("CsoundEngine::messageListDispatcher()");
+//	qDebug("CsoundEngine::messageListDispatcher()");
 	CsoundUserData *ud_local = (CsoundUserData *) data;
 
 	while (ud_local->runDispatcher) {
@@ -1125,7 +1136,7 @@ void CsoundEngine::messageListDispatcher(void *data)
 
 		usleep(ud_local->msgRefreshTime);
 	}
-	qDebug() << "messageListDispatcher quit";
+//	qDebug() << "messageListDispatcher quit";
 }
 
 void CsoundEngine::flushQueues()
@@ -1158,24 +1169,6 @@ void CsoundEngine::queueMessage(QString message)
 	m_messageMutex.unlock();
 }
 
-void CsoundEngine::recordBuffer()
-{
-	if (m_recording) {
-		if (ud->audioOutputBuffer.copyAvailableBuffer(m_recBuffer, m_recBufferSize)) {
-			int samps = m_outfile->write(m_recBuffer, m_recBufferSize);
-			samplesWritten += samps;
-		}
-		else {
-			//       qDebug("CsoundQt::recordBuffer() : Empty Buffer!");
-		}
-		recordTimer.singleShot(20, this, SLOT(recordBuffer()));
-	}
-	else { //Stop recording
-		delete m_outfile;
-		qDebug("Recording stopped. Written %li samples", samplesWritten);
-	}
-}
-
 bool CsoundEngine::isRunning()
 {
 	QMutexLocker locker(&m_playMutex);
@@ -1220,7 +1213,8 @@ void CsoundEngine::breakpointCallback(CSOUND *csound, debug_bkpt_info_t *bkpt_in
 			QVariantList varDetails;
             varDetails << vp->name;
             if (strcmp(vp->typeName, "i") == 0
-                    || strcmp(vp->typeName, "k") == 0) {
+					|| strcmp(vp->typeName, "k") == 0
+					|| strcmp(vp->typeName, "r") == 0) {
                 varDetails << *((MYFLT *) vp->data);
             } else if(strcmp(vp->typeName, "S") == 0) {
                 varDetails << (char *) vp->data;
@@ -1237,23 +1231,26 @@ void CsoundEngine::breakpointCallback(CSOUND *csound, debug_bkpt_info_t *bkpt_in
 		vp = vp->next;
 	}
 	cs->variableMutex.unlock();
-    csoundDebugFreeVariables(csound, vp);
-    csoundDebugFreeInstrInstances(csound, debug_instr);
+	if (bkpt_info->currentOpcode) {
+		cs->m_currentLine.store(bkpt_info->currentOpcode->line);
+	} else {
+		cs->m_currentLine.store(debug_instr->line);
+	}
 
-    debug_instr = csoundDebugGetInstrInstances(csound);
+	debug_instr_t *debug_instr_list = bkpt_info->instrListHead;
 	//Copy active instrument list
 	cs->instrumentMutex.lock();
     cs->m_instrumentList.clear();
-    while (debug_instr) {
+	while (debug_instr_list) {
 		QVariantList instance;
-        instance << debug_instr->p1;
-        instance << QString("%1 %2").arg(debug_instr->p2).arg(debug_instr->p3);
-        instance << debug_instr->kcounter;
+		instance << debug_instr_list->p1;
+		instance << QString("%1 %2").arg(debug_instr_list->p2).arg(debug_instr_list->p3);
+		instance << debug_instr_list->kcounter;
 		cs->m_instrumentList << instance;
-        debug_instr = debug_instr->next;
+		debug_instr_list = debug_instr_list->next;
 	}
-    cs->instrumentMutex.unlock();
-    csoundDebugFreeInstrInstances(csound, debug_instr);
+	cs->instrumentMutex.unlock();
+
 	emit cs->breakpointReached();
 }
 
@@ -1305,6 +1302,20 @@ void CsoundEngine::removeInstrumentBreakpoint(double instr)
 	}
 }
 
+void CsoundEngine::addBreakpoint(int line, int instr, int skip)
+{
+	if (isRunning() && m_debugging) {
+		csoundSetBreakpoint(ud->csound, line, instr, skip);
+	}
+}
+
+void CsoundEngine::removeBreakpoint(int line, int instr)
+{
+	if (isRunning() && m_debugging) {
+		csoundRemoveBreakpoint(ud->csound, line, instr);
+	}
+}
+
 void CsoundEngine::setStartingBreakpoints(QVector<QVariantList> bps)
 {
 	m_startBreakpoints = bps;
@@ -1328,4 +1339,9 @@ QVector<QVariantList> CsoundEngine::getInstrumentList()
 	return outList;
 }
 
+int CsoundEngine::getCurrentLine()
+{
+	return m_currentLine.load();
+}
+
 #endif
