@@ -26,9 +26,9 @@
 #include <QStringList>
 #include <QTimer>
 #include <QFuture>
+#include <QAtomicInt>
 
 #include <csound.hpp>
-#include <sndfile.hh>
 #include <csPerfThread.hpp>
 #include <cwindow.h> // Necessary for WINDAT struct
 
@@ -95,6 +95,7 @@ struct CsoundUserData {
 	QList<QVariant> previousStringOutputValues;
 
 	void *midiBuffer; //Csound Circular Buffer
+	void *virtualMidiBuffer; //Csound Circular Buffer
 
 #ifdef QCS_PYTHONQT
 	PythonConsole *m_pythonConsole;
@@ -146,6 +147,7 @@ public:
 	static int midiOutCloseCb(CSOUND *csound, void *ud);
 	static const char *midiErrorStringCb(int);
 	void queueMidiIn(std::vector<unsigned char> *message);
+	void queueVirtualMidiIn(std::vector<unsigned char> &message);
 	void sendMidiOut(QVector<unsigned char> &message);
 
 	static void makeGraphCallback(CSOUND *csound, WINDAT *windat, const char *name);
@@ -199,11 +201,15 @@ public:
 	void stopDebug();
 	void addInstrumentBreakpoint(double instr, int skip);
 	void removeInstrumentBreakpoint(double instr);
+	void addBreakpoint(int line, int instr, int skip);
+	void removeBreakpoint(int line, int instr);
 	void setStartingBreakpoints(QVector<QVariantList> bps);
 	QVector<QVariantList> getVaribleList();
 	QVector<QVariantList> getInstrumentList();
+	int getCurrentLine();
 	QVector<QVariantList> m_varList;
 	QVector<QVariantList> m_instrumentList;
+	QAtomicInt m_currentLine;
 	QMutex variableMutex;
 	QMutex instrumentMutex;
 	QVector<QVariantList> m_startBreakpoints;
@@ -238,13 +244,6 @@ private:
 
 	CsoundUserData *ud;
 
-	SndfileHandle *m_outfile;
-	long samplesWritten;
-	bool m_recording;
-	MYFLT *m_recBuffer; // for temporary copy of Csound output buffer when recording to file
-	int m_recBufferSize; // size of the record buffer
-	QTimer recordTimer;
-
 	CsoundOptions m_options;
 
 	QVector<ConsoleWidget *> consoles;  // Consoles registered for message printing
@@ -255,7 +254,7 @@ private:
 	QStringList keyPressBuffer; // protected by keyMutex
 	QStringList keyReleaseBuffer; // protected by keyMutex
 
-
+	bool m_recording;
 	QMutex m_playMutex; // To prevent from starting a Csound instance while another is starting or closing
 	QMutex eventMutex;
 	QVector<QString> eventQueue;
@@ -264,8 +263,6 @@ private:
 	int eventQueueSize;
 
 private slots:
-	void recordBuffer();
-	//    void widgetLayoutDestroyed();
 
 signals:
 	void errorLines(QList<QPair<int, QString> >);
