@@ -1152,18 +1152,25 @@ void DocumentPage::registerButton(QuteButton *b)
 
 void DocumentPage::queueMidiIn(std::vector< unsigned char > *message)
 {
-	WidgetLayout *d = m_widgetLayouts[0];
-	unsigned int nBytes = message->size();
-	Q_ASSERT(nBytes < 4);
-	if ( ((d->midiWriteCounter + 1) % QCS_MAX_MIDI_QUEUE) != d->midiReadCounter) {
-		int index = d->midiWriteCounter;
-		d->midiWriteCounter++;
-		d->midiWriteCounter = d->midiWriteCounter % QCS_MAX_MIDI_QUEUE;
-		for (unsigned int i = 0; i < nBytes; i++) {
-			d->midiQueue[index][i] = (int)message->at(i);
-		}
-	}
-	m_csEngine->queueMidiIn(message);
+    WidgetLayout *d = m_widgetLayouts[0];
+    unsigned int nBytes = message->size();
+    if (nBytes > 3 || nBytes < 1) {
+        qDebug() << "MIDI message ignored. nBytes=" << nBytes << " status =" << (int)message->at(0);
+        return;
+    }
+    if ( ((d->midiWriteCounter + 1) % QCS_MAX_MIDI_QUEUE) != d->midiReadCounter) {
+        int index = d->midiWriteCounter;
+        for (unsigned int i = 0; i < nBytes; i++) {
+            d->midiQueue[index][i] = (int)message->at(i);
+        }
+        d->midiWriteCounter = (d->midiWriteCounter + 1) % QCS_MAX_MIDI_QUEUE;
+    }
+    m_csEngine->queueMidiIn(message);
+}
+
+void DocumentPage::queueVirtualMidiIn(std::vector< unsigned char > &message)
+{
+    m_csEngine->queueVirtualMidiIn(message);
 }
 
 void DocumentPage::init(QWidget *parent, OpEntryParser *opcodeTree)
@@ -1255,6 +1262,7 @@ void DocumentPage::stop()
 
 int DocumentPage::record(int format)
 {
+#ifdef	PERFTHREAD_RECORD
 	if (fileName.startsWith(":/")) {
 		QMessageBox::critical(static_cast<QWidget *>(parent()),
 							  tr("CsoundQt"),
@@ -1275,6 +1283,10 @@ int DocumentPage::record(int format)
 	}
 	emit setCurrentAudioFile(recName);
 	return m_csEngine->startRecording(format, recName);
+#else
+	QMessageBox::warning(NULL, tr("Recording not possible"), tr("This version of CsoundQt was not built with recording support."));
+	return 0;
+#endif
 }
 
 void DocumentPage::perfEnded()
