@@ -29,10 +29,25 @@
 #include "highlighter.h"
 #include "texteditor.h"
 
+
 DocumentView::DocumentView(QWidget * parent, OpEntryParser *opcodeTree) :
 	BaseView(parent,opcodeTree)
 {
 	m_autoComplete = true;
+
+	m_hoverWidget = new HoverWidget(m_mainEditor);
+	m_hoverText= new QLabel("Hello", m_hoverWidget);
+	m_hoverText->show();
+	m_hoverText->setStyleSheet(
+				"QLabel {"
+				"border: 1px solid black;"
+//				"border-radius: 4px;"
+				"padding: 2px;"
+				"background-color: #ffffcc;"
+				"};");
+	m_hoverText->setWordWrap(true);
+	m_hoverWidget->hide();
+
 	for (int i = 0; i < editors.size(); i++) {
 		connect(editors[i], SIGNAL(textChanged()), this, SLOT(setModified()));
 		splitter->addWidget(editors[i]);
@@ -47,14 +62,15 @@ DocumentView::DocumentView(QWidget * parent, OpEntryParser *opcodeTree) :
 	setFocusProxy(m_mainEditor);  // for comment action from main application
 	internalChange = false;
 
+
 	//  m_highlighter = new Highlighter();
 
 	connect(m_mainEditor, SIGNAL(textChanged()),
 			this, SLOT(textChanged()));
-	connect(m_mainEditor, SIGNAL(cursorPositionChanged()),
-			this, SLOT(syntaxCheck()));
 	connect(m_mainEditor, SIGNAL(cursorPositionChanged()),
 			this, SLOT(updateContext()));
+	connect(m_mainEditor, SIGNAL(cursorPositionChanged()),
+			this, SLOT(syntaxCheck()));
 	connect(m_mainEditor, SIGNAL(escapePressed()),
 			this, SLOT(escapePressed()));
 	connect(m_mainEditor, SIGNAL(newLine()),
@@ -71,8 +87,10 @@ DocumentView::DocumentView(QWidget * parent, OpEntryParser *opcodeTree) :
 			this, SLOT(exitParameterMode()));
 	connect(m_mainEditor, SIGNAL(enterPressed()),
 			this, SLOT(finishParameterMode()));
-    connect(m_mainEditor, SIGNAL(mouseReleased()),
-            this, SLOT(exitParameterMode()));
+	connect(m_mainEditor, SIGNAL(showParameterInfo()),
+			this, SLOT(showHoverText()));
+	connect(m_mainEditor, SIGNAL(requestParameterModeExit()),
+			this, SLOT(exitParameterMode()));
 
 	//TODO put this for line reporting for score editor
 	//  connect(scoreEditor, SIGNAL(textChanged()),
@@ -97,18 +115,6 @@ DocumentView::DocumentView(QWidget * parent, OpEntryParser *opcodeTree) :
 	setViewMode(0);  // To force a change
 	setAcceptDrops(true);
 
-	m_hoverWidget = new QWidget(m_mainEditor);
-	m_hoverText= new QLabel("Hello", m_hoverWidget);
-	m_hoverText->show();
-	m_hoverText->setStyleSheet(
-				"QLabel {"
-				"border: 1px solid black;"
-//				"border-radius: 4px;"
-				"padding: 2px;"
-				"background-color: #ffffcc;"
-				"};");
-	m_hoverText->setWordWrap(true);
-	m_hoverWidget->hide();
 }
 
 DocumentView::~DocumentView()
@@ -144,12 +150,11 @@ void DocumentView::updateContext()
 	contextStart << "<CsInstruments>" << "<CsScore" << "<CsOptions>";
 	QTextCursor cursor = m_mainEditor->textCursor();
 	cursor.select(QTextCursor::LineUnderCursor);
-	QString line = cursor.selection().toPlainText();
-	int outerContext = 0;
+	QString line = cursor.selection().toPlainText().simplified();
 	while (!cursor.atStart()) {
 		foreach(QString startText, contextStart) {
-			if (line.contains(startText)) {
-				outerContext = contextStart.indexOf(startText) + 1;
+			if (line.startsWith(startText)) {
+				m_currentContext = contextStart.indexOf(startText) + 1;
 				break;
 			}
 		}
@@ -158,7 +163,7 @@ void DocumentView::updateContext()
 		line = cursor.selection().toPlainText();
 		cursor.movePosition(QTextCursor::StartOfBlock);
 	}
-	if (outerContext == 1) { // Instrument section
+	if (m_currentContext == DocumentView::ORC_CONTEXT) { // Instrument section
 		QString endText = "</CsInstruments>";
 		QTextCursor linecursor = cursor;
 		linecursor.select(QTextCursor::LineUnderCursor);
@@ -271,9 +276,10 @@ void DocumentView::nextParameter()
 			cursor.movePosition(QTextCursor::PreviousCharacter, QTextCursor::KeepAnchor);
 		}
 	}
+	m_mainEditor->blockSignals(true); // To avoid triggering the position changed action which hides the text cursor and disables parameter mode
 	m_mainEditor->setTextCursor(cursor);
-	QRect cursorRect = m_mainEditor->cursorRect();
-	updateHoverText(cursorRect.x(), cursorRect.y(), cursor.selectedText());
+	m_mainEditor->blockSignals(false);
+	showHoverText();
 }
 
 void DocumentView::prevParameter()
@@ -305,18 +311,30 @@ void DocumentView::prevParameter()
 			cursor.movePosition(QTextCursor::NextCharacter, QTextCursor::KeepAnchor);
 		}
 	}
+	m_mainEditor->blockSignals(true); // To avoid triggering the position changed action which hides the text cursor and disables parameter mode
 	m_mainEditor->setTextCursor(cursor);
+	m_mainEditor->blockSignals(false);
+	showHoverText();
+}
+
+void DocumentView::showHoverText()
+{
+	QTextCursor cursor  = m_mainEditor->textCursor();
 	QRect cursorRect = m_mainEditor->cursorRect();
 	updateHoverText(cursorRect.x(), cursorRect.y(), cursor.selectedText());
 }
 
+void DocumentView::hideHoverText()
+{
+	m_hoverWidget->hide();
+}
+
 void DocumentView::updateHoverText(int x, int y, QString text)
 {
 	QString displayText = m_currentOpcodeText;
-	if (text.isEmpty()) {
-		return;
+	if (!text.isEmpty()) {
+		displayText.replace(text, "<b>" + text + "</b>");
 	}
-	displayText.replace(text, "<b>" + text + "</b>");
 	m_hoverText->setText(displayText);
 	m_hoverText->adjustSize();
 	QRect textRect = m_hoverText->contentsRect();
@@ -635,19 +653,136 @@ void DocumentView::syntaxCheck()
 	else { //  Split view
 		editor = (TextEditor *) sender();
 	}
+
+	// Clear marked parens
 	QTextCursor cursor = editor->textCursor();
+	if (m_parenspos.first >= 0) {
+		cursor.setPosition(m_parenspos.first);
+		cursor.movePosition(QTextCursor::NextCharacter, QTextCursor::KeepAnchor, 2);
+		QTextCharFormat fmt;
+		editor->blockSignals(true);
+		cursor.setCharFormat(fmt);
+		editor->blockSignals(false);
+		m_parenspos.first = -1;
+	}
+	if (m_parenspos.second >= 0) {
+		cursor.setPosition(m_parenspos.second);
+		cursor.movePosition(QTextCursor::NextCharacter, QTextCursor::KeepAnchor, 2);
+		QTextCharFormat fmt;
+		editor->blockSignals(true);
+		cursor.setCharFormat(fmt);
+		editor->blockSignals(false);
+		m_parenspos.second = -1;
+	}
+
+	QStringList parenStack;
+	bool rightParenMatch = false;
+	// Check for matching parens
+	cursor = editor->textCursor();
+	cursor.movePosition(QTextCursor::PreviousCharacter, QTextCursor::KeepAnchor);
+	QString chr = cursor.selectedText();
+	if (chr == ")") {
+		m_parenspos.second = cursor.position();
+		m_parenspos.first = m_parenspos.second;
+		parenStack.push_back(")");
+		while (m_parenspos.first-- >= 0) {
+			cursor.clearSelection();
+			cursor.movePosition(QTextCursor::PreviousCharacter, QTextCursor::KeepAnchor);
+			QString chr = cursor.selectedText();
+			if (chr == ")") {
+				parenStack.push_back(")");
+			}
+			if (chr == "(") {
+				parenStack.pop_back();
+				if (parenStack.length() == 0) {
+					rightParenMatch = true;
+					break;
+				}
+			}
+		}
+	}
+
+	cursor = editor->textCursor();
+	cursor.movePosition(QTextCursor::NextCharacter, QTextCursor::KeepAnchor);
+	chr = cursor.selectedText();
+	if (chr == "(") {
+		m_parenspos.first = cursor.position() - 1;
+		m_parenspos.second = m_parenspos.first + 1;
+		parenStack.push_back(")");
+		int charCount = editor->document()->characterCount();
+		while (m_parenspos.second < charCount) {
+			cursor.clearSelection();
+			cursor.movePosition(QTextCursor::NextCharacter, QTextCursor::KeepAnchor);
+			QString chr = cursor.selectedText();
+			if (chr == "(") {
+				parenStack.push_back("(");
+			}
+			if (chr == ")") {
+				parenStack.pop_back();
+				if (parenStack.length() == 0) {
+					rightParenMatch = true;
+					break;
+				}
+			}
+			m_parenspos.second++;
+		}
+	}
+
+	// Now highlight the parens
+	if(m_parenspos.first >=0) {
+		cursor.setPosition(m_parenspos.first);
+		cursor.movePosition(QTextCursor::NextCharacter, QTextCursor::KeepAnchor);
+		QTextCharFormat fmt;
+		if (rightParenMatch) {
+			fmt.setBackground(QBrush(Qt::lightGray));
+		} else {
+			fmt.setBackground(QBrush(Qt::magenta));
+		}
+		editor->blockSignals(true);
+		cursor.setCharFormat(fmt);
+		editor->blockSignals(false);
+	}
+
+	if(m_parenspos.second >=0) {
+		cursor.setPosition(m_parenspos.second);
+		cursor.movePosition(QTextCursor::NextCharacter, QTextCursor::KeepAnchor);
+		QTextCharFormat fmt;
+		if (rightParenMatch) {
+			fmt.setBackground(QBrush(Qt::lightGray));
+		} else {
+			fmt.setBackground(QBrush(Qt::magenta));
+		}
+		editor->blockSignals(true);
+		cursor.setCharFormat(fmt);
+		editor->blockSignals(false);
+	}
+
+	cursor = editor->textCursor();
+
+	cursor.movePosition(QTextCursor::EndOfWord, QTextCursor::MoveAnchor);
 	cursor.movePosition(QTextCursor::StartOfLine, QTextCursor::KeepAnchor);
-	QStringList words = cursor.selectedText().split(QRegExp("\\b"));
+	QStringList words = cursor.selectedText().split(QRegExp("\\b"),
+													QString::SkipEmptyParts);
+	bool showHover = false;
 	for(int i = 0; i < words.size(); i++) {
 		QString word = words[words.size() - i - 1];
 		if (m_opcodeTree->isOpcode(word)) {
 			QString syntax = m_opcodeTree->getSyntax(word);
 			if(!syntax.isEmpty()) {
+				emit(opcodeSyntaxSignal(syntax));
 				m_currentOpcodeText = syntax;
-				return;
+				if (i == 0 && editor->textCursor().hasSelection()) {
+					showHover = true;
+				}
+				break;
 			}
 		}
 	}
+	if (showHover) {
+		showHoverText();
+	} else {
+		hideHoverText();
+	}
 }
 
 void DocumentView::textChanged()
@@ -659,8 +794,6 @@ void DocumentView::textChanged()
 	TextEditor *editor = m_mainEditor;
 	unmarkErrorLines();
 
-//	parameterButton->setVisible(false);
-
 	if (m_mode == EDIT_CSOUND_MODE || m_mode == EDIT_ORC_MODE) {  // CSD or ORC mode
 		if (m_autoComplete) {
 			QTextCursor cursor = editor->textCursor();
@@ -824,6 +957,8 @@ void DocumentView::escapePressed()
 			m_mainEditor->moveCursor(QTextCursor::NextCharacter);
 			m_mainEditor->moveCursor(QTextCursor::PreviousCharacter);
 			exitParameterMode();
+		} else if (m_hoverText->isVisible()){
+			hideHoverText();
 		} else {
 			emit closeExtraPanels();
 		}
@@ -840,6 +975,8 @@ void DocumentView::finishParameterMode()
 		if (m_mainEditor->getParameterMode()) {
 			killToEnd();
 			exitParameterMode();
+		} else {
+			hideHoverText();
 		}
 	} else {
 		qDebug() << "finishParameterMode() not implemented for split view";
@@ -854,7 +991,7 @@ void DocumentView::exitParameterMode()
 	} else {
 		qDebug() << "exitParameterMode() not implemented for split view";
 	}
-	m_hoverWidget->hide();
+	hideHoverText();
 //	parameterButton->setVisible(false);
 }
 
@@ -1701,3 +1838,13 @@ void MySyntaxMenu::keyPressEvent(QKeyEvent * event)
 	QMenu::keyPressEvent(event);
 }
 
+HoverWidget::HoverWidget(QWidget *parent) :
+	QWidget(parent)
+{
+
+}
+
+void HoverWidget::mousePressEvent(QMouseEvent *ev)
+{
+	this->hide();
+}
