@@ -25,6 +25,10 @@
 
 #ifdef USE_QT5
 #include <QtWidgets>
+#ifdef USE_QT_GT_53
+#include <QQuickWidget>
+#include <QQuickItem>
+#endif
 #else
 #include <QtGui>
 #endif
@@ -153,6 +157,7 @@ public slots:
 	void pause(int index = -1);
 	void stop(int index = -1);
 	void stopAll();
+	void stopAllOthers();
 	void markStopped();
 	void perfEnded();
 	void render();
@@ -164,7 +169,9 @@ public slots:
 	void closeExtraPanels(); // to close help and console panels when esc is pressed in the editor
 	//    void updateWidgets();
 	void openExample();
+	void openTemplate();
 	void logMessage(QString msg);
+    void statusBarMessage(QString message);
 	//    void registerLiveEvent(QWidget *e);
 	void evaluateCsound(QString code = QString());
 	void breakpointReached();
@@ -217,8 +224,10 @@ private slots:
 	void setHelpEntry();
 	void setFullScreen(bool full);
 	void showDebugger(bool show);
+	void showVirtualKeyboard(bool show);
 	void splitView(bool split);
 	void showMidiLearn();
+	void virtualMidiIn(QVariant on, QVariant note, QVariant channel, QVariant velocity);
 	void openManualExample(QString fileName);
 	void openExternalBrowser(QUrl url = QUrl());
 	void openPdfFile(QString name);
@@ -257,9 +266,9 @@ private slots:
 	void stopDebugger();
 	void pauseDebugger();
 	void continueDebugger();
-	void addBreakpoint(int line, int skip);
+	void addBreakpoint(int line, int instr, int skip);
 	void addInstrumentBreakpoint(double instr, int skip);
-	void removeBreakpoint(int line);
+	void removeBreakpoint(int line, int instr);
 	void removeInstrumentBreakpoint(double instr);
 #endif
 private:
@@ -275,12 +284,13 @@ private:
 	void fillScriptsSubMenu(QDir dir, QMenu *m, int depth);
 	void fillEditScriptsSubMenu(QDir dir, QMenu *m, int depth);
 	void createToolBars();
+    void createStatusBar();
 	void readSettings();
 	void writeSettings(QStringList openFiles=QStringList(), int lastIndex = 0);
 	void clearSettings();
 	int execute(QString executable, QString options);
 	//    bool saveCurrent();
-	void makeNewPage(QString fileName, QString text);
+	bool makeNewPage(QString fileName, QString text);
 	bool loadCompanionFile(const QString &fileName);
 	void setCurrentFile(const QString &fileName);
 	QString strippedName(const QString &fullFileName);
@@ -308,6 +318,9 @@ private:
 	DockHelp *helpPanel;
 	WidgetPanel *widgetPanel;  // Dock widget, for containing the widget layout
 	QDockWidget *m_scratchPad;
+#ifdef USE_QT_GT_53
+	QQuickWidget *m_virtualKeyboard;
+#endif
 	//    QString m_widgetClipboard;
 	Inspector *m_inspector;
 #ifdef QCS_DEBUGGER
@@ -325,6 +338,7 @@ private:
 
 	QMenu *fileMenu;
 	QMenu *recentMenu;
+	QMenu *templateMenu;
 	QMenu *editMenu;
 	QMenu *controlMenu;
 	QMenu *viewMenu;
@@ -387,6 +401,7 @@ private:
 #ifdef QCS_DEBUGGER
 	QAction *showDebugAct;
 #endif
+	QAction *showVirtualKeyboardAct;
 	QAction *midiLearnAct;
 	QAction *splitViewAct;
 
